/* BdfServer_API - Copyright (c) 2008-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.api.ui.options;


/**
 *
 * @author Vincent Calame
 */
public interface InputOption {

    public final static String WIDTH_SMALL = "small";
    public final static String WIDTH_MEDIUM = "medium";
    public final static String WIDTH_LARGE = "large";
    public final static String INPUT_TEXT = "text";
    public final static String INPUT_SELECT = "select";
    public final static String INPUT_CHECK = "check";
    public final static String INPUT_RADIO = "radio";
    public final static String INPUT_NOTEDITABLE = "noteditable";
    public final static String INPUT_HIDDEN = "hidden";
    public final static String INPUT_SUGGESTION = "suggestion";
    public final static String INPUT_FICHESTYLE = "fichestyle";

    public String getInputType();

    public String getWidthType();

    public int getRows();

    public static String checkInputType(String inputType) {
        switch (inputType) {
            case INPUT_TEXT:
            case INPUT_SELECT:
            case INPUT_CHECK:
            case INPUT_RADIO:
            case INPUT_NOTEDITABLE:
            case INPUT_HIDDEN:
            case INPUT_SUGGESTION:
            case INPUT_FICHESTYLE:
                return inputType;
            default:
                throw new IllegalArgumentException("wrong value :" + inputType);
        }
    }

    public static String checkWidthType(String widthType) {
        switch (widthType) {
            case WIDTH_SMALL:
            case WIDTH_MEDIUM:
            case WIDTH_LARGE:
                return widthType;
            default:
                try {
                int size = Integer.parseInt(widthType);
                if (size < 10) {
                    return WIDTH_SMALL;
                }
                if (size < 50) {
                    return WIDTH_MEDIUM;
                }
                return WIDTH_LARGE;
            } catch (NumberFormatException nfe) {
                throw new IllegalArgumentException("wrong value : " + widthType);
            }
        }
    }

}
