/* FichothequeLib_Tools - Copyright (c) 2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.exportation.access;

import net.fichotheque.exportation.access.AccessDef;
import net.fichotheque.selection.FichothequeQueries;
import net.fichotheque.tools.selection.FichothequeQueriesBuilder;
import net.mapeadores.util.attr.Attributes;
import net.mapeadores.util.misc.DefBuilder;
import net.mapeadores.util.text.Labels;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class AccessDefBuilder extends DefBuilder {

    private final String name;
    private final FichothequeQueriesBuilder customFichothequeQueriesBuilder = new FichothequeQueriesBuilder();
    private boolean publicAccess = false;
    private String tableExportName = "";
    private String selectionDefName = "";

    public AccessDefBuilder(String name) {
        this(name, null);
    }

    public AccessDefBuilder(String name, Attributes initAttributes) {
        super(initAttributes);
        if (!StringUtils.isTechnicalName(name, true)) {
            throw new IllegalArgumentException("Wrong syntax");
        }
        this.name = name;
    }

    public FichothequeQueriesBuilder getCustomFichothequeQueriesBuilder() {
        return customFichothequeQueriesBuilder;
    }

    public AccessDefBuilder setPublic(boolean publicAccess) {
        this.publicAccess = publicAccess;
        return this;
    }

    public AccessDefBuilder setTableExportName(String tableExportName) {
        this.tableExportName = StringUtils.nullTrim(tableExportName);
        return this;
    }

    public AccessDefBuilder setSelectionDefName(String selectionDefName) {
        this.selectionDefName = StringUtils.nullTrim(selectionDefName);
        return this;
    }

    public AccessDef toAccessDef() {
        Labels titleLabels = toLabels();
        Attributes attributes = toAttributes();
        return new InternalAccessDef(name, publicAccess, tableExportName, selectionDefName, customFichothequeQueriesBuilder.toFichothequeQueries(), titleLabels, attributes);
    }

    public static AccessDefBuilder init(String name) {
        return new AccessDefBuilder(name);
    }

    public static AccessDefBuilder init(String name, Attributes initAttributes) {
        return new AccessDefBuilder(name, initAttributes);
    }


    private static class InternalAccessDef implements AccessDef {

        private final String name;
        private final boolean publicAccess;
        private final String tableExportName;
        private final String selectionDefName;
        private final FichothequeQueries customFichothequeQueries;
        private final Labels titleLabels;
        private final Attributes attributes;


        private InternalAccessDef(String name, boolean publicAccess, String tableExportName, String selectionDefName, FichothequeQueries customFichothequeQueries, Labels titleLabels, Attributes attributes) {
            this.name = name;
            this.publicAccess = publicAccess;
            this.tableExportName = tableExportName;
            this.selectionDefName = selectionDefName;
            this.customFichothequeQueries = customFichothequeQueries;
            this.titleLabels = titleLabels;
            this.attributes = attributes;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public Labels getTitleLabels() {
            return titleLabels;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

        @Override
        public boolean isPublic() {
            return publicAccess;
        }

        @Override
        public String getTableExportName() {
            return tableExportName;
        }

        @Override
        public String getSelectionDefName() {
            return selectionDefName;
        }

        @Override
        public FichothequeQueries getCustomFichothequeQueries() {
            return customFichothequeQueries;
        }

    }

}
